% Copyright (C) Emily Stephen 2014


clear variables

addpath('tools/')
addpath('networks/')
addpath('simulate/')
addpath('plot/')
addpath('plot/export_fig/')
addpath('plot/rgb2cm/')

wa = warning('off','MATLAB:MKDIR:DirectoryExists');

%% Set up Sims

master_sim.N = 9; % # electrodes
master_sim.Fs = 1200;
master_sim.ntrials = 100;
master_sim.T = master_sim.Fs*(master_sim.ntrials*6);

% pink noise
master_sim.temp_corr = 0.02;

% baseline correlations
master_sim.baseline_band = [2 50];
master_sim.baseline_total_gain = .5;

% trial correlations
master_sim.trial_band = [8 25];
master_sim.trial_window = 'gaussian';

master_sim.trial_electrode_gains{1} = [1 0 0  0  0  0  0  0  1; ...
                                       0 1 0  0  0  0  0  1  0;...
                                       0 0 1  1  0  0  0  0  0];
master_sim.trial_electrode_gains{2} = [1 1 1  0  0  0  0  0  0; ...
                                       0 0 0  1  0  0  1  0  0;...
                                       0 0 0  0  1  0  0  1  0;...
                                       0 0 0  0  0  1  0  0  1];

master_sim.trial_onsets_s = (master_sim.ntrials*4):2:(master_sim.ntrials*6-1);
master_sim.silence_onsets_s = 0:2:(master_sim.ntrials*4-1);
master_sim.trial_dur_s = 1;

trial_gains = [0 0.26 0.63 1.2];
baseline_corr_gains = [1 1/2 1/4];

nBL = length(baseline_corr_gains);
nTR = length(trial_gains);

ind = 1;

% Sim3B*T*: Pink + White + Baseline + Trial
% Case where baseline is zero, varying SNR
gB = 0;
for i=1:nTR,
    gT = trial_gains(i);
    subjects{ind} = ['SIM03_B' num2str(gB,'%1.2f') 'T' num2str(gT,'%1.2f')];
    sim = master_sim;
    sim.white_gain = 0.1; % std(white) = white_gain * std(pink)
    sim.baseline_corr_gain = gB; % std(baseline) = baseline_gain * std(pink)
    sim.trial_gain = gT; % std(trials) = trial_gain * std(pink)
    sims(ind) = sim;
    ind = ind+1;
end


% Sim3B*T*: Pink + White + Baseline + Trial
% Fix SNR, vary baseline correlations
gT = 1;
for i=1:nBL,
    gB = baseline_corr_gains(i);
    subjects{ind} = ['SIM03_B' num2str(gB,'%1.2f') 'T' num2str(gT,'%1.2f')];
    sim = master_sim;
    sim.white_gain = 0.1; % std(white) = white_gain * std(pink)
    sim.baseline_corr_gain = gB; % std(baseline) = baseline_gain * std(pink)
    sim.trial_gain = gT; % std(trials) = trial_gain * std(pink)
    sims(ind) = sim;
    ind = ind+1;
end

%% Run Simulations
reftype = 'Physical';
processing = 'below30Hz';

nsims = length(subjects);

for i=1:nsims,
    subject = subjects{i};
    sim = sims(i);

    simulatedData_ramps(subject,reftype,{processing},sim,false,true);
end

%% Infer Networks
nettypes = {'absZeroLagCorr','CanonicalCorrelation'};
dyntypes = {'BA','Dynamic'};
swtOuterBS_Null = 'same';
strFileAppend = '_demo';

Nbsreps_outer = 100;
Nbsreps = 1000;

for i=1:length(nettypes),
    FuncNetwork_main(subjects,{reftype},{processing},nettypes(i),dyntypes,'Nbsreps_outer',Nbsreps_outer,'Nbsreps',Nbsreps,'blnFitNullBS',false,'ba_corrlag_s',0,'dy_corrlag_s',0,'strFileAppend',strFileAppend,'swtOuterBS_Null',swtOuterBS_Null)
end

%% Plot Results

blnsave = true;
swtCI = 'sterr';

C_azc{1} = [0 0 0 0 0 0 0 0 1;...
            0 0 0 0 0 0 0 1 0;...
            0 0 0 1 0 0 0 0 0;...
            0 0 1 0 0 0 0 0 0;...
            0 0 0 0 0 0 0 0 0;...
            0 0 0 0 0 0 0 0 0;...
            0 0 0 0 0 0 0 0 0;...
            0 1 0 0 0 0 0 0 0;...
            1 0 0 0 0 0 0 0 0];
C_azc{2} = [0 1 1 0 0 0 0 0 0;...
            1 0 1 0 0 0 0 0 0;...
            1 1 0 0 0 0 0 0 0;...
            0 0 0 0 0 0 1 0 0;...
            0 0 0 0 0 0 0 1 0;...
            0 0 0 0 0 0 0 0 1;...
            0 0 0 1 0 0 0 0 0;...
            0 0 0 0 1 0 0 0 0;...
            0 0 0 0 0 1 0 0 0];
C_cc{1} = [0 1 1 ;...
           1 0 0 ;...
           1 0 0 ];
C_cc{2} = [0 0 0 ;...
           0 0 1 ;...
           0 1 0 ];

% Individual subject plots
for s=1:nsims,
    subject = subjects{s};
    for n=1:length(nettypes),
        nettype = nettypes{n};
        if strcmpi(nettype,'absZeroLagCorr'),
            dynYlim = [0 0.4];
            trueafternet = C_azc;
        else
            dynYlim = [0 1];
            trueafternet = C_cc;
        end

        Frontiers_plots(subject,reftype,processing,nettype,'strFileAppend',strFileAppend,'trueafternet',trueafternet,'blnSave',blnsave,'swtCI',swtCI,'dynYlim',dynYlim);%,'proc',proc);
    end
end

% Pan Subject Density figs
trueafternet = C_azc;
dynYlim = [0 0.4];
pansubjplot_density(subjects(1:nTR),reftype,processing,nettypes{1},'strFileAppend',strFileAppend,'strSaveFilePrepend','SNR_','trueafternet',trueafternet,'blnSave',blnsave,'blnObsHeight',true,'Yl',dynYlim,'swtCI',swtCI);
pansubjplot_density(subjects(nTR+1:nsims),reftype,processing,nettypes{1},'strFileAppend',strFileAppend,'strSaveFilePrepend','corrat_','trueafternet',trueafternet,'blnSave',blnsave,'blnObsHeight',true,'Yl',dynYlim,'swtCI',swtCI);

%%
warning(wa);