% Copyright (C) Emily Stephen 2014


clear variables

addpath('tools/')
addpath('networks/')
addpath('simulate/')
addpath('plot/')
addpath('plot/export_fig/')
addpath('plot/rgb2cm/')

wa(1) = warning('off','FRONTERS_PLOTS:NoDynFile');
wa(2) = warning('off','MATLAB:MKDIR:DirectoryExists');

%% Set up Sim

reftype = 'Physical';
processing = 'below30Hz';

sim.N = 10; % # electrodes
sim.Fs = 1200;
sim.ntrials = 100;
sim.T = sim.Fs*(sim.ntrials*6);
sim.temp_corr = 0.02;
sim.baseline_band = [2 50];
sim.baseline_corr_gain = 0;
sim.trial_band = [8 25];
sim.trial_electrode_gains = [ 1  0  0  0  0   1  0  0  0  0;...
                              0  1  0  0  0   0  1  0  0  0;...
                              0  0  1  0  0   0  0  1  0  0;...
                              0  0  0  1  0   0  0  0  1  0;...
                              0  0  0  0  1   0  0  0  0  1;...
                              1  0  0  0  0   0  1  0  0  0;...
                              0  1  0  0  0   0  0  1  0  0;...
                              0  0  1  0  0   0  0  0  1  0;...
                              0  0  0  1  0   0  0  0  0  1;...
                              0  0  0  0  1   1  0  0  0  0;...
                              1  0  0  0  0   0  0  1  0  0;...
                              0  1  0  0  0   0  0  0  1  0;...
                              0  0  1  0  0   0  0  0  0  1;...
                              0  0  0  1  0   1  0  0  0  0;...
                              0  0  0  0  1   0  1  0  0  0;...
                              1  0  0  0  0   0  0  0  1  0;...
                              0  1  0  0  0   0  0  0  0  1;...
                              0  0  1  0  0   1  0  0  0  0;...
                              0  0  0  1  0   0  1  0  0  0;...
                              0  0  0  0  1   0  0  1  0  0;...
                              1  0  0  0  0   0  0  0  0  1;...
                              0  1  0  0  0   1  0  0  0  0;...
                              0  0  1  0  0   0  1  0  0  0;...
                              0  0  0  1  0   0  0  1  0  0;...
                              0  0  0  0  1   0  0  0  1  0];

regions(1) =  struct('name','R1','electrodes',{{'1','2','3','4','5'}},'color',[255 0 0]/255); % aMFg/46,9
regions(2) =  struct('name','R2','electrodes',{{'6','7','8','9','10'}},'color',[0 255 0]/255); % aMFg/46,9

sim.trial_onsets_s = (sim.ntrials*4):2:(sim.ntrials*6-1);
sim.silence_onsets_s = 0:2:(sim.ntrials*4-1);
sim.trial_dur_s = 1;

sim.white_gain = 0.1; % std(white) = white_gain * std(pink)
sim.baseline_total_gain = 0; % std(baseline) = baseline_gain * std(pink)

sim.trial_window = 'gaussian';

C_azc = [0 0 0 0 0 1 1 1 1 1;...
         0 0 0 0 0 1 1 1 1 1;...
         0 0 0 0 0 1 1 1 1 1;...
         0 0 0 0 0 1 1 1 1 1;...
         0 0 0 0 0 1 1 1 1 1;...
         1 1 1 1 1 0 0 0 0 0;...
         1 1 1 1 1 0 0 0 0 0;...
         1 1 1 1 1 0 0 0 0 0;...
         1 1 1 1 1 0 0 0 0 0;...
         1 1 1 1 1 0 0 0 0 0];
C_cc = [0 1;1 0];

sim.trial_gain = 1;
subject = 'SIM_example1';

%% Run Sim
nettypes = {'absZeroLagCorr','CanonicalCorrelation'};
swtOuterBS_Null = 'same';
strFileAppend = '_demo';

Nbsreps_outer = 100;
Nbsreps = 1000;

simulatedData_ramps(subject,reftype,{processing},sim,false,true,regions);

%% Infer Networks
FuncNetwork_main({subject},{reftype},{processing},nettypes,{'BA'},'Nbsreps_outer',Nbsreps_outer,'Nbsreps',Nbsreps,'blnFitNullBS',false,'ba_corrlag_s',0,'dy_corrlag_s',0,'strFileAppend',strFileAppend,'swtOuterBS_Null',swtOuterBS_Null)

%% Plot Results
blnsave = true;
swtCI = 'sterr';

for j=1:length(nettypes),
        if strcmpi(nettypes{j},'absZeroLagCorr'),
            dynYlim = [0 0.4];
            trueafternet = C_azc;
        else
            dynYlim = [0 1];
            trueafternet = C_cc;
        end
        [w(j),~,ba_C{j}] = Frontiers_plots(subject,reftype,processing,nettypes{j},'trueafternet',trueafternet,'strFileAppend',strFileAppend,'blnSave',false,'blnJustProc',true,'dynYlim',dynYlim,'swtCI',swtCI);
end

loadpath = ['Output_Processed Correlation Networks/' reftype '/' processing '/' nettypes{j} '/wOuterBootstrap/'];
l=load([loadpath 'workspace' '_S' subject '_BA' strFileAppend '.mat'],'regions');
regions = l.regions;

p_edge_azc = mean(ba_C{1}(:,:,2,:),4);
p_edge_cc = mean(ba_C{2}(:,:,2,:),4);

f=figure(1);
set(f,'position',[135   478   506   282]);
cmap = flipud(colormap(gray(100)));
clf reset; set(f,'Color','w')
subplot(2,3,1),cla reset; set(gca, 'Color', 'none'); 
sndf_plotnetwork(w(1).dsc_utterances,w(1).brain,C_azc,'regions',regions,'offset',[-50 0 0],'bln2D',false);grid off;axis off;colorbar('off')
subplot(2,3,2),cla reset; set(gca, 'Color', 'none'); 
sndf_plotnetwork(w(1).dsc_utterances,w(1).brain,w(1).agg(2).C,'regions',regions,'offset',[-50 0 0],'bln2D',false);grid off;axis off;colorbar('off')
subplot(2,3,3),cla reset; set(gca, 'Color', 'none'); 
sndf_plotnetwork(w(1).dsc_utterances,w(1).brain,triu(ones(sim.N,sim.N),1),p_edge_azc,'colorlims',[0 1],'regions',regions,'offset',[-50 0 0],'bln2D',false,'cmap',cmap);grid off;axis off;colorbar('off')
subplot(2,3,4),cla reset; set(gca, 'Color', 'none'); grid off;
sndf_plotnetwork_regions(w(2).dsc_utterances,w(2).brain,C_cc,regions,'offset',[-50 0 0],'bln2D',false);grid off;axis off;colorbar('off')
subplot(2,3,5),cla reset; set(gca, 'Color', 'none'); grid off;
sndf_plotnetwork_regions(w(2).dsc_utterances,w(2).brain,w(2).agg(2).C,regions,'offset',[-50 0 0],'bln2D',false);grid off;axis off;colorbar('off')
subplot(2,3,6),cla reset; set(gca, 'Color', 'none'); grid off;
sndf_plotnetwork_regions(w(2).dsc_utterances,w(2).brain,triu(ones(2,2),1),regions,p_edge_cc,'colorlims',[0 1],'offset',[-50 0 0],'bln2D',false,'cmap',cmap);grid off;axis off;colorbar('off')

if blnsave,
    savepath = ['Output_Plots/'];
    mkdir(savepath)
    filename = [savepath 'S' subject '_summary'];
    rgb2cm(f)
    export_fig([filename '.pdf'],'-pdf','-m1', f)  
    export_fig([filename '.png'],'-png','-r1200','-m1', f)  
end

%%
warning(wa);
