% Copyright (C) Emily Stephen 2014


clear variables

addpath('tools/')
addpath('networks/')
addpath('simulate/')
addpath('plot/')
addpath('plot/export_fig/')
addpath('plot/rgb2cm/')

wa(1) = warning('off','FRONTERS_PLOTS:NoDynFile');
wa(2) = warning('off','MATLAB:MKDIR:DirectoryExists');

%% Set up Sims

reftype = 'Physical';
processings = {'below30Hz'};

master_sim.N = 9; % # electrodes
master_sim.Fs = 1200;
master_sim.ntrials = 100;
master_sim.T = master_sim.Fs*(master_sim.ntrials*6);
master_sim.temp_corr = 0.02;
master_sim.baseline_band = [2 50];
master_sim.baseline_corr_gain = 0;
master_sim.trial_band = [8 25];
master_sim.trial_window = 'gaussian';
master_sim.trial_electrode_gains = [];
master_sim.trial_onsets_s = (master_sim.ntrials*4):2:(master_sim.ntrials*6-1);
master_sim.silence_onsets_s = 0:2:(master_sim.ntrials*4-1);
master_sim.trial_dur_s = 1;

master_sim.white_gain = 0.1;
master_sim.baseline_total_gain = 0;
master_sim.trial_gain = 1;

ind = 1;

subjects{ind} = ['SIM_example2a'];
sims(ind) = master_sim;
sims(ind).trial_electrode_gains = [0 0 0   1  -1  0  -1  1  0];
     
ind = ind + 1;
subjects{ind} = ['SIM_example2b'];
sims(ind) = master_sim;
sims(ind).trial_electrode_gains = [0 0 0   1  0  0   1  0  0;...
                                   0 0 0   0  1  0   0 -1  0];
                               
%% Run Sims

nsims = length(subjects);

for i=1:nsims,
    subject = subjects{i};
    sim = sims(i);
    
    simulatedData_ramps(subject,reftype,processings,sim,false,true);
end

%% Create cnt and dsc files with averaged regions
processings_out = cellfun(@(x) [x '_aveReg'],processings,'UniformOutput',false);

for i=1:length(subjects),
    subject = subjects{i};
    processing = processings{1};
    processing_out = processings_out{1};
    
    % Load cnt, dsc, and regions
    cnt = load(['Output_Preprocessed Data/SUBJECT' subject '/' reftype '/' processing '/ECOGS001_' processing '_cnt.mat']);
    dsc_utterances = load(['Output_Preprocessed Data/SUBJECT' subject '/' reftype '/' processing '/ECOGS001_manualUtt_wcnt_dsc.mat']);
    l = load(['Output_Preprocessed Data/SUBJECT' subject  '/SUBJECT' subject '_regions.mat']);
    regions = l.regions;

    % Average signals by region
    cnt_out = averageRegions(cnt,regions);
    dsc_out = averageRegions(dsc_utterances,regions);

    % Save new cnt, dsc
    savepath = ['Output_Preprocessed Data/SUBJECT' subject '/' reftype '/' processing_out];
    mkdir(savepath)
    cntfile = [savepath filesep 'ECOGS001' '_' processing_out '_cnt.mat'];
    dscfile = [savepath '/ECOGS001_manualUtt_wcnt_dsc.mat'];
    save(cntfile,'-struct','cnt_out','-v7.3');
    save(dscfile,'-struct','dsc_out','-v7.3');
end


%% Run absZeroLagCorr networks on the averaged data and canonical correlation on the unaveraged data
reftypes = {'Physical'};
processings = {'below30Hz_aveReg','below30Hz'};
dyntypes = {'BA'};
swtOuterBS_Null = 'same';
blnJackknife = true;

Nbsreps_outer = 100;
Nbsreps = 1000;
strFileAppend = '_demo';

FuncNetwork_main(subjects,reftypes,processings,{'absZeroLagCorr'},dyntypes,'Nbsreps_outer',Nbsreps_outer,'Nbsreps',Nbsreps,'blnFitNullBS',false,'ba_corrlag_s',0,'dy_corrlag_s',0,'strFileAppend',strFileAppend,'swtOuterBS_Null',swtOuterBS_Null)

FuncNetwork_main(subjects,reftypes,{'below30Hz'},{'CanonicalCorrelation'},dyntypes,'Nbsreps_outer',Nbsreps_outer,'Nbsreps',Nbsreps,'blnFitNullBS',false,'ba_corrlag_s',0,'dy_corrlag_s',0,'strFileAppend',strFileAppend,'swtOuterBS_Null',swtOuterBS_Null)

%% Plot results
swtCI = 'sterr';
nettypes = {'absZeroLagCorr','CanonicalCorrelation'};
blnsave = true;

C_azc{1} = [0 0 0 0 0 0 0 0 0;...
            0 0 0 0 0 0 0 0 0;...
            0 0 0 0 0 0 0 0 0;...
            0 0 0 0 1 0 1 1 0;...
            0 0 0 1 0 0 1 1 0;...
            0 0 0 0 0 0 0 0 0;...
            0 0 0 1 1 0 0 1 0;...
            0 0 0 1 1 0 1 0 0;...
            0 0 0 0 0 0 0 0 0];
C_cc{1} = [0 0 0 ;...
           0 0 1 ;...
           0 1 0 ];
C_azc{2} = [0 0 0 0 0 0 0 0 0;...
            0 0 0 0 0 0 0 0 0;...
            0 0 0 0 0 0 0 0 0;...
            0 0 0 0 0 0 1 0 0;...
            0 0 0 0 0 0 0 1 0;...
            0 0 0 0 0 0 0 0 0;...
            0 0 0 1 0 0 0 0 0;...
            0 0 0 0 1 0 0 0 0;...
            0 0 0 0 0 0 0 0 0];
C_cc{2} = [0 0 0 ;...
           0 0 1 ;...
           0 1 0 ];

for s=1:length(subjects),
    subject = subjects{s};
    nettype = 'absZeroLagCorr';
    for p=1:length(processings),
        processing = processings{p};
        if strcmpi(processing,'below30Hz'),
            dynYlim = [0 0.4];
            trueafternet = C_azc{s};
        else
            dynYlim = [0 1];
            trueafternet = C_cc{s};
        end
        [w(s,p),~,ba_C{s,p}] = Frontiers_plots(subject,reftypes{1},processing,nettype,'trueafternet',trueafternet,'strFileAppend',strFileAppend,'blnSave',false,'blnJustProc',true,'dynYlim',dynYlim,'swtCI',swtCI);
    end
end

for s=1:length(subjects),
    subject = subjects{s};
    dynYlim = [0 1];
    trueafternet = C_cc{s};
    nettype = 'CanonicalCorrelation';
    processing = 'below30Hz';
    [w2(s),~,ba_C2{s}] = Frontiers_plots(subject,reftypes{1},processing,nettype,'trueafternet',trueafternet,'strFileAppend',strFileAppend,'blnSave',false,'blnJustProc',true,'dynYlim',dynYlim,'swtCI',swtCI);
end

cmap = flipud(colormap(gray(100)));
for s=1:length(subjects),
    subject = subjects{s};
    f=figure(1);
    clf reset; set(f,'Color','w');set(f,'position',[ 538   276   707   603])
    
    brain = w2(s).brain;
    du = w2(s).dsc_utterances;
    N = 9;
    nR = 3;
    
    p_edge_azc = mean(ba_C{s,2}(:,:,2,:),4);
    p_edge_ac = mean(ba_C{s,1}(:,:,2,:),4);
    p_edge_cc = mean(ba_C2{s}(:,:,2,:),4);
    
    loadpath = ['Output_Processed Correlation Networks/' reftypes{1} '/' processings{2} '/' nettypes{2} '/wOuterBootstrap/'];
    l = load([loadpath 'workspace' '_S' subjects{s} '_BA' strFileAppend '.mat'],'regions');
    regions = l.regions;

    subplot(3,3,1),cla reset; set(gca, 'Color', 'none'); 
    sndf_plotnetwork(du,brain,C_azc{s},'regions',regions);grid off;axis off;colorbar('off')
    subplot(3,3,2),cla reset; set(gca, 'Color', 'none'); 
    sndf_plotnetwork(du,brain,w(s,2).agg(2).C,'regions',regions);grid off;axis off;colorbar('off')
    subplot(3,3,3),cla reset; set(gca, 'Color', 'none'); 
    sndf_plotnetwork(du,brain,triu(ones(N,N,1)),p_edge_azc,'colorlims',[0 1],'regions',regions,'cmap',cmap);grid off;axis off;colorbar('off')

    subplot(3,3,4),cla reset; set(gca, 'Color', 'none'); grid off;
    sndf_plotnetwork_regions(du,brain,C_cc{s},regions);grid off;axis off;colorbar('off')
    subplot(3,3,5),cla reset; set(gca, 'Color', 'none'); grid off;
    sndf_plotnetwork_regions(du,brain,w(s,1).agg(2).C,regions);grid off;axis off;colorbar('off')
    subplot(3,3,6),cla reset; set(gca, 'Color', 'none'); grid off;
    sndf_plotnetwork_regions(du,brain,triu(ones(nR,nR),1),regions,p_edge_ac,'colorlims',[0 1],'cmap',cmap);grid off;axis off;colorbar('off')

    subplot(3,3,7),cla reset; set(gca, 'Color', 'none'); grid off;
    sndf_plotnetwork_regions(du,brain,C_cc{s},regions,'offset',[-50 0 0],'bln2D',false);grid off;axis off;colorbar('off')
    subplot(3,3,8),cla reset; set(gca, 'Color', 'none'); grid off;
    sndf_plotnetwork_regions(du,brain,w2(s).agg(2).C,regions);grid off;axis off;colorbar('off')
    subplot(3,3,9),cla reset; set(gca, 'Color', 'none'); grid off;
    sndf_plotnetwork_regions(du,brain,triu(ones(nR,nR),1),regions,p_edge_cc,'colorlims',[0 1],'cmap',cmap);grid off;axis off;colorbar('off')

    if blnsave,
        savepath = ['Output_Plots/'];
        mkdir(savepath)
        filename = [savepath 'S' subject '_summary'];
        rgb2cm(f)
        export_fig([filename '.pdf'],'-pdf','-m1', f)  
        export_fig([filename '.png'],'-png','-r1200','-m1', f)  
    else
        pause
    end
end

%%
warning(wa);