% Copyright (C) Emily Stephen 2014
% Correlation based functional connectivity script

function FuncNetwork_main(varargin)
ip = inputParser;
ip.addRequired('subjects',@iscell);
ip.addRequired('reftypes',@iscell)
ip.addRequired('processings',@iscell)
ip.addRequired('nettypes',@iscell)
ip.addRequired('dyntypes',@iscell) % 'BA','Dynamic'
ip.addParamValue('ba_corrlag_s',0.2,@isnumeric)
ip.addParamValue('dy_corrlag_s',0.05,@isnumeric)
ip.addParamValue('Nbsreps',500,@isnumeric);
ip.addParamValue('blnFitNullBS',false,@islogical) % whether to approximate the null distributions (Fisher transformed) by a gaussian
ip.addParamValue('Nbsreps_outer',0,@isnumeric); % set to zero for no outer bootstrap
ip.addParamValue('swtOuterBS_Null','same',@(x) strcmp('same',x)||strcmp('resampleWInds',x)||strcmp('resample',x)) % for Outer Bootstrap: 'same' (the same null as the observed, single sampling),'resampleWInds' (resample null for each obs sample and apply inds),'resample' (reestimate null once for all obs samples, double sampling)
ip.addParamValue('strFileAppend','',@ischar)
ip.addParamValue('blnsave',true,@islogical)
ip.parse(varargin{:});

subjects = ip.Results.subjects;
reftypes = ip.Results.reftypes;
processings = ip.Results.processings;
nettypes = ip.Results.nettypes;
dyntypes = ip.Results.dyntypes;
ba_corrlag_s = ip.Results.ba_corrlag_s; 
dy_corrlag_s = ip.Results.dy_corrlag_s;
Nbsreps = ip.Results.Nbsreps;
blnFitNullBS = ip.Results.blnFitNullBS;
Nbsreps_outer = ip.Results.Nbsreps_outer;
swtOuterBS_Null = ip.Results.swtOuterBS_Null;
strFileAppend = ip.Results.strFileAppend;
blnsave = ip.Results.blnsave;

total_tic = tic;

% Before/After Parameters
ba_trial_s = 0.5;

% Dynamic Paramters
dy_net_binlen_s = 0.2;


%% Loop
for s=1:length(subjects),
    subject = subjects{s};
    ecogrun = '';
    silence_buffer_s = 0;
    
    brain = load(['Output_Preprocessed Data/AMC_' subject '_brain.mat']);
    
    for r=1:length(reftypes),
        reftype = reftypes{r};
        
        for p=1:length(processings),
            processing = processings{p};
            
            
            %% Load
            cnt = load(['Output_Preprocessed Data/SUBJECT' subject '/' reftype '/' processing '/ECOGS001' ecogrun '_' processing '_cnt.mat']);
            dsc_utterances = load(['Output_Preprocessed Data/SUBJECT' subject '/' reftype '/' processing '/ECOGS001' ecogrun '_manualUtt_wcnt_dsc.mat']);
            dsc_silences = load(['Output_Preprocessed Data/SUBJECT' subject  '/ECOGS001' ecogrun '_silenceOnOff_dsc.mat']);
            
            Fs = dsc_utterances.SegSampFreq;
            N = size(dsc_utterances.SegValues,3);
            ChLbl = dsc_utterances.ChLbl;
            nonsets = length(dsc_utterances.EvtTimes);
            disp(['   Number of utterances: ' num2str(nonsets)])
            
            for d=1:length(dyntypes),
                dyntype = dyntypes{d};
                disp(['Subject: ' subject ', Ref: ' reftype ', Proc: ' processing ', Dyn: ' dyntype])
                
                switch dyntype,
                    case 'BA'
                        maxLags = floor(ba_corrlag_s*Fs); 
                        [agg_ecog bsecog names Nwin nnets] = ba_setup(dsc_utterances,dsc_silences,cnt,N,silence_buffer_s,ba_trial_s);

                    case 'Dynamic'
                        maxLags = floor(dy_corrlag_s*Fs); 
                        switch processing
                            case 'below30Hz'
                                dy_net_step = 1; 
                            otherwise
            %                     dy_net_step = 20; 
                                dy_net_step = 1; 
                        end

                        [agg_ecog bsecog names Nwin nnets start_mask end_mask] = dy_setup(dsc_utterances,dsc_silences,cnt,N,Fs,silence_buffer_s,dy_net_binlen_s,dy_net_step);
                end

                for n=1:length(nettypes),
                    nettype = nettypes{n};

                    close all
                    
                    % Set up save file
                    if blnsave,
                        if Nbsreps_outer>0,
                            savepath = ['Output_Processed Correlation Networks/' reftype '/' processing '/' nettype '/wOuterBootstrap/'];
                        else
                            if blnFitNullBS,
                                savepath = ['Output_Processed Correlation Networks/' reftype '/' processing '/' nettype '/'];
                            else
                                savepath = ['Output_Processed Correlation Networks/' reftype '/' processing '/' nettype '/EmpPvals/'];
                            end
                        end
                        mkdir(savepath)
                        
                        filename = ['workspace' '_S' subject '_' dyntype strFileAppend '.mat'];
                        
                        savename = [savepath filename];

                        save(savename,'subject','reftype','processing','nettype','dyntype','silence_buffer_s','Nbsreps','ecogrun','brain','dsc_utterances','Fs','maxLags','N','nonsets','Nwin','names','nnets','names','Nbsreps_outer','swtOuterBS_Null','-v7.3')
                        
                        % Dynamic variables
                        if strcmpi(dyntype,'Dynamic'),
                            save(savename,'dy_net_step','start_mask','end_mask','-append','-v7.3')
                        end
                    else
                        savename = '';
                    end
                    
                    % for corr nets, divide by std
                    if any(strcmpi(nettype,{'maxAbsCorr','zeroLagCorr','absZeroLagCorr','CanonicalCorrelation'})),
                        for i=1:nnets,
                            agg_ecog{i} = bsxfun(@rdivide,agg_ecog{i},std(agg_ecog{i},1,1)); 
                        end
                        bsecog = bsxfun(@rdivide,bsecog,std(bsecog,1,1)); 
                    end

                    % load region file for canonical statistics
                    if any(strcmpi(nettype,{'CanonicalCorrelation','CanonicalCoherence'})),
                        l = load(['Output_Preprocessed Data/SUBJECT' subject  '/SUBJECT' subject '_regions.mat']);
                        regions = l.regions;
                        
                        if blnsave,
                            save(savename,'regions','-append','-v7.3')
                        end
                    end

                    %% calculate networks
                    params.nettype_short = nettype;
                    params.nonsets = nonsets;
                    params.ChLbl = ChLbl;
                    params.maxLags = maxLags;
                    params.Nbsreps = Nbsreps;
                    params.Nbsreps_outer = Nbsreps_outer;
                    params.swtOuterBS_Null = swtOuterBS_Null;
                    params.savename = savename;
                    params.blnFitNullBS = blnFitNullBS;

                    if blnsave,
                        save(savename,'params','-append','-v7.3')
                    end

                    switch nettype,
                        case {'maxAbsCorr','zeroLagCorr','absZeroLagCorr'}
                            computeNets_corr(bsecog,agg_ecog,params);
                        case {'CanonicalCorrelation'}
                            computeNets_ccorr(bsecog,agg_ecog,regions,params);
                    end

                end
            end
        end
    end
end

toc(total_tic)
