% Copyright (C) Emily Stephen 2014
function [xhat_bootstrap] = ccor_bootstrap_sample(varargin)
% bsecog_regions: cell{Nr} of (p x nstrials x T)
% xhat_bootstrap: (Nbsreps x Nr x Nr)
ip = inputParser;
ip.addRequired('bsecog_regions',@iscell)
ip.addRequired('Nbsreps',@isnumeric)
ip.addRequired('nonsets',@isnumeric)
ip.addParamValue('inds',[],@isnumeric);
ip.addParamValue('blnDoubleSample',false,@islogical)
ip.parse(varargin{:});

bsecog_regions = ip.Results.bsecog_regions;
Nbsreps = ip.Results.Nbsreps;
nonsets = ip.Results.nonsets;
inds = ip.Results.inds;
blnDoubleSample = ip.Results.blnDoubleSample;

Nr = length(bsecog_regions);
[~,Nbtrials,T] = size(bsecog_regions{1});

blnApplyInds = ~isempty(inds);

if nonsets>Nbtrials
    warning('BOOTSTRAPSAMPLE:IndsWarning',...
      ['nonsets (' num2str(nonsets) ') > Nstrials (' num2str(Nbtrials) '). # replicates will not match.'])
end

xhat_bootstrap = nan(Nbsreps,Nr,Nr);
parfor i=1:Nbsreps,    
    % create samples of nonsets trials selected randomly with replacement
    % from all silence, before, after trials for all electrodes
    samp = randi(Nbtrials,[nonsets,1]);
    
    if blnDoubleSample,
        samp = samp(randi(nonsets,[nonsets,1]));
    end
    
    if blnApplyInds,
        samp = samp(inds);
    end
    
    sample = cell(1,Nr); % cell{Nr} of (p x ntrials x T)
    for r=1:Nr,
        reg = bsecog_regions{r};
        sample{r} = reg(:,samp,:);
    end
    
    xhat_bootstrap(i,:,:) = ccor_helper(sample);
end

