% Copyright (C) Emily Stephen 2014
% Calculate Correlations from time-domain data
function [xhat] = ccor_helper(agg_ecog_regions,sample)
%   agg_ecog_regions: cell{Nr} of (p x ntrials x T)
%   xhat: (Nr x Nr)

    
Nr = length(agg_ecog_regions);
[~,ntrials,T] = size(agg_ecog_regions{1});

if nargin>1,
    ntrials = length(sample);
end

k = ntrials*T;

% % subtract off time mean
% for r=1:Nr,
%     trials = agg_ecog_regions{r};
%     agg_ecog_regions{r} = bsxfun(@minus,trials,mean(trials,3)); %Remove the time mean.
% end

xhat = zeros(Nr,Nr);
for r1 = 1:Nr,
    for r2 = (r1+1):Nr,
        td1 = agg_ecog_regions{r1};
        td2 = agg_ecog_regions{r2};
        
        if nargin>1,
            td1 = td1(:,sample,:);
            td2 = td2(:,sample,:);
        end
        
        % concatenate trials to get (p x ntrials*T)
        p = size(td1,1);
        q = size(td2,1);
        cat1 = nan(p,k);
        cat2 = nan(q,k);
        for i=1:p,
            cat1(i,:) = reshape(td1(i,:,:),1,k);
        end
        for i=1:q,
            cat2(i,:) = reshape(td2(i,:,:),1,k);
        end
        
        % use canonical coherence code, where k is ntrials*T and nF is 1.
        xhat(r1,r2,:) = compute_canonical_coherence(cat1,cat2);
    end
end
    

