% Copyright (C) Emily Stephen 2014
function [tf_compiled,regions_out] = compile_regions(tf,ChLbl,regions,numD)
% tf: 
%   (N x K x nstrials x nF) 
%   OR 
%   (N x k x nF), or, for the time domain, (N x ntrials x T)
% ChLbl: cell array from cnt data
% regions: cell array of cell arrays allocating electrodes to regions
% tf_compiled: 
%   cell{Nr} of (p x K x ntrials x nF)
%   OR 
%   cell{Nr} of (p x k x nF), or, for the time domain, cell{Nr} of (p x ntrials x T)

Nr = length(regions);
N = length(ChLbl);


sz = size(tf);
if sz(1)~=N,
    % check that the number of electrodes in ChLbl matches tf
    error('the first dimension of tf (N) should match the length of ChLbl')
end

tf_compiled = cell(Nr,1);
regions_out = regions;
for r=1:Nr,
    [els,ichlbl,ireg] = intersect(ChLbl,regions(r).electrodes);
    regions_out(r).electrodes = els;
    regions_out(r).name = regions(r).name;
    
    switch numD,
        case 4
            tf_compiled{r} = tf(ichlbl,:,:,:);
        case 3
            tf_compiled{r} = tf(ichlbl,:,:);
        otherwise
            error('tf should be 3d or 4d')
    end
            
end

% remove empty regions
ps = cellfun(@(x) size(x,1),tf_compiled);
empties = ps==0;

tf_compiled = tf_compiled(~empties);
regions_out = regions_out(~empties);

% check that all electrodes are used, none twice
used = [regions_out(:).electrodes];
if length(intersect(used,ChLbl))~=N,
    error('Some electrodes do not have regions')
end

if length(used)~=N,
    error('Some electrodes are used more than once')
end
