% Copyright (C) Emily Stephen 2014
function [] = computeNets_ccorr(varargin)
ip = inputParser;
ip.addRequired('bsecog',@isnumeric)
ip.addRequired('agg_ecog',@iscell)
ip.addRequired('regions',@isstruct)
ip.addRequired('params',@isstruct)
ip.parse(varargin{:});

bsecog = ip.Results.bsecog;
agg_ecog = ip.Results.agg_ecog;
regions = ip.Results.regions;
params = ip.Results.params;

nettype_short = params.nettype_short;
nonsets = params.nonsets;
ChLbl = params.ChLbl;
Nbsreps = params.Nbsreps;
Nbsreps_outer = params.Nbsreps_outer;
swtOuterBS_Null = params.swtOuterBS_Null;
savename = params.savename;
blnFitNullBS = params.blnFitNullBS;

%% Compile regions together
% bsecog_regions: cell{Nr} of (p x nstrials x T)
% agg_ecog_regions: cell{2} of cell{Nr} of (p x ntrials x T)
disp(['      Compiling regions...'])
[bsecog_regions,regions] = compile_regions(permute(bsecog,[2 3 1]),ChLbl,regions,3);
agg_ecog_regions = cellfun(@(x) compile_regions(permute(x,[2 3 1]),ChLbl,regions,3),agg_ecog,'UniformOutput',false);

%% Compute p-values
% Calculate Canonical correlation for bootstraps and networks
% xhat_true: cell{2} of (Nr x Nr)
% xhat_bootstrap: (Nbsreps x Nr x Nr)
disp(['      Calculating ' nettype_short ' for bootstrap...'])
xhat_bootstrap = ccor_bootstrap_sample(bsecog_regions,Nbsreps,nonsets);

[agg_rho,xhat_true,xhat_bs_mean,xhat_bs_std] = OneSampleNet(agg_ecog_regions,xhat_bootstrap,1:nonsets,blnFitNullBS);

if ~isempty(savename),
    % Null fit
    if blnFitNullBS,
        save(savename,'xhat_bs_mean','xhat_bs_std','-append','-v7.3')
    end

    % baseline distribution
    save(savename,'xhat_true','xhat_bootstrap','-append','-v7.3');

end

%% Compute Network
disp(['      Calculating ' nettype_short ' networks...'])
tic
agg = network_loop_rho(agg_rho);
toc

if ~isempty(savename),
    % Observed Network
    save(savename,'agg','-append','-v7.3');
end
clear agg

%% Outer Bootstrap
switch swtOuterBS_Null,
    case 'same',
        xhat_bootstrap_outer = xhat_bootstrap;
    case 'resampleWInds',
        xhat_bootstrap_outer = [];
    case 'resample',
        disp(['      Calculating bootstrap samples for outer bootstrap networks...'])
        xhat_bootstrap_outer = ccor_bootstrap_sample(bsecog_regions,Nbsreps,nonsets,'blnDoubleSample',true);
end
blnResampleNull = strcmp(swtOuterBS_Null,'resampleWInds');

agg_ebs = cell(1,Nbsreps_outer);
ebs_samples = zeros(nonsets,Nbsreps_outer);
%     for ibs_out=1:Nbsreps_outer,
parfor ibs_out=1:Nbsreps_outer,
    tic
    disp(['        Outer bootstrap sample #' num2str(ibs_out) '...'])
    % create bootstrap sample of trials
    % Sample from the set of trials with replacement
    ebs_sample = randi(nonsets,[nonsets,1]);

    % calculate xhat_ebs
    if blnResampleNull,
        xhat_bootstrap_ebs = ccor_bootstrap_sample(bsecog_regions,Nbsreps,nonsets,'inds',ebs_sample);
    else
        xhat_bootstrap_ebs = xhat_bootstrap_outer;
    end

    [agg_rho_ebs] = OneSampleNet(agg_ecog_regions,xhat_bootstrap_ebs,ebs_sample,blnFitNullBS);

    agg_ebs{ibs_out} = network_loop_rho(agg_rho_ebs);

    ebs_samples(:,ibs_out) = ebs_sample;
    toc
end

if ~isempty(savename),
    % Outer Bootstrap
    if Nbsreps_outer>0,
        save(savename,'agg_ebs','ebs_samples','-append','-v7.3')
    end
    
    % Outer Null
    if strcmp(swtOuterBS_Null,'resample')
        save(savename,'xhat_bootstrap_outer','-append','-v7.3')
    end
end
clear agg_ebs ebs_samples xhat_bootstrap_outer

end

function [agg_rho,xhat,xhat_bs_mean,xhat_bs_std] = OneSampleNet(agg_ecog_regions,xhat_bootstrap,trialInds,blnFitNullBS)
    xhat = cellfun(@(x) ccor_helper(x,trialInds),agg_ecog_regions,'UniformOutput',false);
    
    if blnFitNullBS,
        % estimate mean and std error of bootstraps
        % xhat_bs_mean: (N x N), (N x N x nF), (Nr x Nr), or (Nr x Nr x nF)
        % xhat_bs_std: (N x N), (N x N x nF), (Nr x Nr), or (Nr x Nr x nF)
        xhat_bs_F = atanh(xhat_bootstrap);
        xhat_bs_mean = squeeze(mean(xhat_bs_F,1));
        xhat_bs_std = squeeze(std(xhat_bs_F,1,1));

        agg_rho = cellfun(@(xhat) compute_rho_OneSample(xhat,xhat_bs_mean,xhat_bs_std),xhat,'UniformOutput',false);
    else
        agg_rho = cellfun(@(xhat) compute_rho_OneSample(xhat,xhat_bootstrap),xhat,'UniformOutput',false);
        xhat_bs_mean = [];
        xhat_bs_std = [];
    end
end



