% Copyright (C) Emily Stephen 2014
function [CC,u1,v1,Q_XY] = compute_canonical_coherence(X,Y)
% X: matrix of tapered fourier coefficients (p x k x nf)
% Y: matrix of tapered fourier coefficients (q x k x nf)
% NOTE: Canonical correlation uses the same code, where k is ntrials*ntimepoints
%       and nf is 1.

[p,k,nf] = size(X);
[q,k2,nf2] = size(Y);

if k~=k2 || nf~=nf2,
    error('Dimensions of X and Y must match')
end

Q_XY = nan(p,q,nf);
CC = nan(1,nf);
u1 = nan(p,nf);
v1 = nan(q,nf);
for fi=1:nf,
    % 2) Compute SVDs of X=U_X*Sig_X*V_X*' and Y=U_Y*Sig_Y*V_Y*'
    [U_X,~,V_X] = svd(X(:,:,fi),'econ');
    [U_Y,~,V_Y] = svd(Y(:,:,fi),'econ');

    % 3) Calculate Q_XY = U_X*V_X'*V_Y*U_Y
    Q_XY(:,:,fi) = U_X*V_X'*V_Y*U_Y';
    
    % 4) Calculate the SVD of Q_XY = U*Sig*V*'
    [U,S,V] = svd(Q_XY(:,:,fi));
    
    CC(fi) = S(1,1); % first singular value
    u1(:,fi) = U(:,1); % first left singular vector
    v1(:,fi) = V(:,1); % first right singular vector
end
