% Copyright (C) Emily Stephen 2014
function [cc_out,lags,ac_out] = compute_xcorrs_new(trials,maxLags,blnAveTrials)

% trials: time x electrodes x trials

[Nwin,N,Ntrials] = size(trials);

% norm_trials = trials - repmat(mean(trials,1),[Nwin 1 1]); %Remove the time mean.
% norm_trials = norm_trials ./ repmat(std(norm_trials,[],1),[Nwin 1 1]); %Set time stdev to 1.
norm_trials = trials; % already normed in main

% Calculate xcorr (using fft) for all trials, between all pairs of
% electrodes
winSizeM = maxLags*2+1;
ac_all = nan(N,Ntrials);
cc_unnormed = nan(N,winSizeM,N,Ntrials);
for o=1:Ntrials,      
    % calculate xcorr for all pairs of electrodes for this trial
    X = norm_trials(:,:,o)';                                      %Call the data X.

    %Zero pad.
    pad = zeros(N, maxLags);      %The padding has length of maxLags.
    X = [pad, X, pad];            %Add padding to beginning and end of data.


    % Compute the autocorrelation at zero lag
    ac = sum(X.*X,2);               %  N x 1
    ac_all(:,o) = ac;

    % Compute the cross-correlation
    Xf = fft(X,[],2);
    X3 = repmat(Xf,[1,1,N]);                     %  make 3-dim [N,winSize,N] ...
    cc = ifft(permute(X3,[3,2,1]).*conj(X3), [], 2);      %  use FTs ...
    cc_unnormed(:,:,:,o) = cc(:,[(end-maxLags+1:end) (1:maxLags+1)],:);
    
end

if blnAveTrials,
    % average xcorr over trials
    cc_out = average_xcorr(cc_unnormed,ac_all,1:Ntrials);
else
    % return the xcorr for all trials separately
    cc_out = cc_unnormed;
    ac_out = ac_all;
end

lags = -maxLags:maxLags;
