% Copyright (C) Emily Stephen 2014
function [bs_cc samps] = corr_bootstrap_sample(varargin)
ip = inputParser;
ip.addRequired('cc_baseline',@isnumeric)
ip.addRequired('ac_baseline',@isnumeric)
ip.addRequired('Nbsreps',@isnumeric)
ip.addRequired('nonsets',@isnumeric)
ip.addParamValue('inds',[],@isnumeric);
ip.addParamValue('blnDoubleSample',false,@islogical)
ip.parse(varargin{:});

cc_baseline = ip.Results.cc_baseline;
ac_baseline = ip.Results.ac_baseline;
Nbsreps = ip.Results.Nbsreps;
nonsets = ip.Results.nonsets;
inds = ip.Results.inds;
blnDoubleSample = ip.Results.blnDoubleSample;

[N,winSizeM,~,Nbtrials] = size(cc_baseline);

if any(size(cc_baseline)~=[N,winSizeM,N,Nbtrials]),
    error('BOOTSTRAPSAMPLE:CCerror',...
      'Size of cc input must be N x winSizeM x N x Nbtrials');
end

blnApplyInds = ~isempty(inds);

blnOutputSamps = nargout>1;

if nonsets>Nbtrials,
    warning('BOOTSTRAPSAMPLE:IndsWarning',...
      ['nonsets (' num2str(nonsets) ') > Nbtrials (' num2str(Nbtrials) '). # replicates will not match.'])
end

% Calculate xcorr for bootstrap samples
bs_cc = cell(1,Nbsreps);
if blnOutputSamps, samps = zeros(nonsets,Nbsreps); end
parfor i=1:Nbsreps,
%     disp(num2str(nargin))
      
    % create samples of nonsets trials selected randomly with replacement
    % from all silence, before, after trials for all electrodes
    samp = randi(Nbtrials,[nonsets,1]);
    
    if blnDoubleSample,
        samp = samp(randi(nonsets,[nonsets,1]));
    end
    
    if blnApplyInds,
        samp = samp(inds);
    end

    % average xcorr over trials
    bs_cc{i} = average_xcorr(cc_baseline,ac_baseline,samp);
    
    if blnOutputSamps, samps(:,i) = samp; end
end




