% Copyright (C) Emily Stephen 2014
function [baw,dyw,ba_C,density,BCa]=Frontiers_plots(varargin)
ip = inputParser;
ip.addRequired('subject',@ischar);
ip.addRequired('reftype',@ischar)
ip.addRequired('processing',@ischar)
ip.addRequired('nettype',@ischar)
ip.addOptional('trueafternet',[],@(x) isnumeric(x)||iscell(x))
ip.addParamValue('blnSave',true,@islogical) 
ip.addParamValue('blnCombineDynamic',true,@islogical)
ip.addParamValue('strFileAppend','',@ischar)
ip.addParamValue('dynYlim',[],@isnumeric)
ip.addParamValue('blnJustProc',false,@islogical)
ip.addParamValue('swtCI','quantile',@(x) any(strcmp(x,{'quantile','sterr','BCa'})))
ip.addParamValue('proc',[],@isstruct)
ip.parse(varargin{:});

subject = ip.Results.subject;
reftype = ip.Results.reftype;
processing = ip.Results.processing;
nettype = ip.Results.nettype;
trueafternet = ip.Results.trueafternet;
blnsave = ip.Results.blnSave;
blnCombineDynamic = ip.Results.blnCombineDynamic;
strFileAppend = ip.Results.strFileAppend;
dynYlim = ip.Results.dynYlim;
swtCI = ip.Results.swtCI;
proc = ip.Results.proc;

close all

loadpath = ['Output_Processed Correlation Networks/' reftype '/' processing '/' nettype '/wOuterBootstrap/'];
baw=load([loadpath 'workspace' '_S' subject '_BA' strFileAppend '.mat'],'agg','names','dsc_utterances','brain','Fs','Nbsreps','Nbsreps_outer');
try
    dyw=load([loadpath 'workspace' '_S' subject '_Dynamic' strFileAppend '.mat'],'agg','names','start_mask','end_mask','Fs','Nbsreps','Nbsreps_outer');
    blnDyn = true;
catch err
    if (strcmp(err.identifier,'MATLAB:load:couldNotReadFile'))
        blnDyn = false;
        warning('FRONTERS_PLOTS:NoDynFile','Dynamic file not found. Computing only BA plots')
        dyw = [];
    else
        rethrow(err)
    end
end


Nbsreps_outer = baw.Nbsreps_outer;
ba_nnets = length(baw.agg);
N = size(baw.agg(1).C,1);
if blnDyn,
    dy_nnets = length(dyw.agg);
    dy_ta = (dyw.start_mask+dyw.end_mask+1)'/2/dyw.Fs;
end


blnCanon = strcmpi(nettype,'CanonicalCorrelation') || ~isempty(strfind(processing,'aveReg'));
switch nettype,
    case 'CanonicalCorrelation'
        nettype_nn = 'CCorr';
    case 'zeroLagCorr'
        nettype_nn = 'Corr';
    case 'absZeroLagCorr'
        nettype_nn = 'AbsCorr';
end

l = load(['Output_Preprocessed Data/SUBJECT' subject  '/SUBJECT' subject '_regions.mat']);
regions = l.regions;

blntruecomp = ~isempty(trueafternet);
if blntruecomp,
    cmap_bin = [255 0 0; 0 0 0]/255;
    
    if ~iscell(trueafternet),
        tan = trueafternet;
        clear trueafternet
        trueafternet{1} = zeros(size(tan));
        trueafternet{2} = tan;
    end
    ntrialparts = 2;
    
    for i=1:ntrialparts,
        truemult{i} = trueafternet{i}*2-1; % 1 for true edges, -1 for false edges
        truedensity(i) = sum(sum(trueafternet{i}))./(N*(N-1));
    end
else
    cmap_bin = 'jet';
end


if ~isempty(strfind(processing,'aveReg')),
    strFolderAppend = '_averaged';
elseif ~isempty(strfind(processing,'var')),
    strFolderAppend = ['_' regexp(processing,'var\d*\.*\d*','match','once')];
elseif ~strcmp(swtCI,'quantile')
    strFolderAppend = ['_' swtCI];
else
    strFolderAppend = '';
end

saveFolder = 'Output_Plots/';

nets_to_plot = [21 51 111 141];
savepath = [saveFolder nettype strFolderAppend strFileAppend '/'];

if blnsave,
    mkdir(savepath)
end

nntoplot = length(nets_to_plot);

%% extract C and density from agg_ebs variables
if isempty(proc),
    baw_ebs=load([loadpath 'workspace' '_S' subject '_BA' strFileAppend '.mat'],'agg_ebs');

    ba_C = nan(N,N,ba_nnets,Nbsreps_outer);
    ba_density = nan(ba_nnets,Nbsreps_outer);
    for i=1:Nbsreps_outer,
        for j = 1:ba_nnets,
            ba_C(:,:,j,i) = baw_ebs.agg_ebs{i}(j).C;
            ba_density(j,i) = baw_ebs.agg_ebs{i}(j).density;
        end
    end
    clear baw_ebs

    if blnDyn,
        dyw_ebs=load([loadpath 'workspace' '_S' subject '_Dynamic' strFileAppend '.mat'],'agg_ebs');
        dy_density = nan(dy_nnets,Nbsreps_outer);
        for i=1:Nbsreps_outer,
            for j=1:dy_nnets,
                dy_density(j,i) = dyw_ebs.agg_ebs{i}(j).density;
            end
        end
        clear dyw_ebs
    end

else
    ba_C = proc.ba_C;
    ba_density = proc.density.ba_ebs;
    dy_density = proc.density.dy_ebs;
end
if nargout>3,
    density.ba_ebs = ba_density;
    density.ba = [baw.agg.density];
    if blnDyn,
        density.dy_ebs = dy_density;
        density.dy = [dyw.agg.density];    
    end
end
if ip.Results.blnJustProc,
    return
end

%% Before/After Binary

% BA
f=figure(1);
clf reset; 
for i=1:ba_nnets,
    subplot(1,ba_nnets,i)
    cla reset; set(gca, 'Color', 'none'); grid off;
    
    if blntruecomp,
        tm = truemult{i};
        cvals = baw.agg(i).C .* tm;
    else
        cvals = [];
    end
        
    if blnCanon,
        sndf_plotnetwork_regions(baw.dsc_utterances,baw.brain,baw.agg(i).C,regions,cvals,'cmap',cmap_bin,'colorlims',[-1 1],'blnLabelReg',false);
    else
        sndf_plotnetwork(baw.dsc_utterances,baw.brain,baw.agg(i).C,cvals,'regions',regions,'cmap',cmap_bin,'colorlims',[-1 1],'blnLabelElec',false)
    end
    grid off;axis off;colorbar('off');axis square
%     title(['Binary: ' w.ba_names(i)])
end
set(f,'Position',[ 135   508   592   252])
set(f,'Color','w')
if blnsave,
    filename = [savepath 'S' subject '_' nettype_nn 'BinaryNet'];
    rgb2cm(f)
    export_fig([filename '.pdf'],'-pdf','-m1', f)  
    export_fig([filename '.png'],'-png','-r1200','-m1', f)  
end

%% Before/After P(edge)
cmap = flipud(colormap(gray(100)));
% cmap = [ones(size(cmap));cmap];

f=figure(2);
set(f,'Position',[ 135   508   592   252])
set(f,'Color','w')
for i=1:ba_nnets,
    subplot(1,2,i)
    
    p_edge = mean(ba_C(:,:,i,:),4);
     
    if blnCanon,
        sndf_plotnetwork_regions(baw.dsc_utterances,baw.brain,triu(ones(N,N,1)),regions,p_edge,'cmap',cmap,'colorlims',[0 1],'blnLabelReg',false);
    else
        sndf_plotnetwork(baw.dsc_utterances,baw.brain,triu(ones(N,N,1)),p_edge,'regions',regions,'cmap',cmap,'colorlims',[0 1],'blnLabelElec',false)    
    end
    grid off;axis off;colorbar('off');axis square
%     title(['P(edge) from bootstrap: ' w.ba_names(i)])
end
if blnsave,
    filename = [savepath 'S' subject '_' nettype_nn 'P_edge'];
    rgb2cm(f)
    export_fig([filename '.pdf'],'-pdf','-m1', f)  
    export_fig([filename '.png'],'-png','-r1200','-m1', f)  
%     pause
end

%% Density vs time, with confidence intervals
if blnDyn,

    fig = figure(5);
    if blnCombineDynamic,
        subplot(2,nntoplot,1:nntoplot)
        set(fig,'Position',[492   545   753   400])
    else
        set(fig,'Position',[360   454   583   224])
    end
    
    obs_dens = [dyw.agg.density]';
    
    lh=plot(dy_ta,obs_dens,'k','linewidth',2); % plot observed density
    if blntruecomp,
        line([dy_ta(1) 0],ones(2,1)*truedensity(1),'color','k','linewidth',2,'linestyle','--');
        line([0 dy_ta(end)],ones(2,1)*truedensity(2),'color','k','linewidth',2,'linestyle','--');
    end
    BCa = struct([]);
    switch swtCI,
        case 'quantile'
            bs_ci = quantile(dy_density',[0.025 0.975])';
        case 'sterr'
            sterr = std(dy_density,[],2);
            bs_ci = [obs_dens+norminv(0.025)*sterr obs_dens+norminv(0.975)*sterr];
    end
    ci_h=fill_ci(lh,dy_ta,bs_ci(:,1),bs_ci(:,2),0.2);
    set(ci_h,'FaceColor',[.8 .8 .8])
    uistack(ci_h,'bottom')

    if ~isempty(dynYlim),
        ylim(dynYlim);
    end

    yl = ylim;
    for j=1:length(nets_to_plot),
        line(ones(2,1)*dy_ta(nets_to_plot(j)),yl,'color','k','linewidth',2)
    end
    ylim(yl)
    xlim(dy_ta([1 end]))
    set(gca,'LineWidth',2)
    set(gca,'Layer','top')
    set(gca,'Box','off')
    set(gca,'FontSize',16)
    set(fig,'Color','w')

    if blnsave && ~blnCombineDynamic,
        filename = [savepath 'S' subject '_' nettype_nn 'DensityVsTime'];
        rgb2cm(fig)
        export_fig([filename '.pdf'],'-pdf','-m1', fig)  
        export_fig([filename '.png'],'-png','-r1200','-m1', fig)  
    %     pause
    end
end

%% Selected networks on the brain
if blnDyn,
    for s=1:length(nets_to_plot)
        i = nets_to_plot(s);

        if blnCombineDynamic,
            subplot(2,nntoplot,nntoplot+s)
            cla reset
        else
            f=figure(6);
            clf reset; 
        end
        set(gca, 'Color', 'none'); grid off;

        if blntruecomp,
            if dy_ta(i)>0,
                tm = truemult{2};
            else
                tm = truemult{1};
            end
            cvals = dyw.agg(i).C .* tm;
        else
            cvals = [];
        end

        if blnCanon,
            sndf_plotnetwork_regions(baw.dsc_utterances,baw.brain,dyw.agg(i).C,regions,cvals,'cmap',cmap_bin,'colorlims',[-1 1],'blnLabelReg',false);
        else
            sndf_plotnetwork(baw.dsc_utterances,baw.brain,dyw.agg(i).C,cvals,'regions',regions,'cmap',cmap_bin,'colorlims',[-1 1],'blnLabelElec',false)
        end
        grid off;axis off;colorbar('off');axis square

        disp(dy_ta(i))

        if blnsave && ~blnCombineDynamic,
            filename = [savepath 'S' subject '_' nettype_nn 'NetOnBrain_' num2str(dy_ta(i))];
            set(f,'Color','w')
            rgb2cm(f)
            export_fig([filename '.pdf'],'-pdf','-m1', f)    
            export_fig([filename '.png'],'-png','-r1200','-m1', f)  
    %         pause
        elseif ~blnsave,
            pause
        end
    end

    if blnsave && blnCombineDynamic,
        filename = [savepath 'S' subject '_' nettype_nn 'DynamicSummary'];
        set(fig,'Color','w')
        rgb2cm(fig)
        export_fig([filename '.pdf'],'-pdf','-m1', fig)   
        export_fig([filename '.png'],'-png','-r1200','-m1', fig)  
    %     pause
    end
end

