% Copyright (C) Emily Stephen 2014
function pansubjplot_density(varargin)
ip = inputParser;
ip.addRequired('subjects',@iscell);
ip.addRequired('reftype',@ischar)
ip.addRequired('processing',@ischar)
ip.addRequired('nettype',@ischar)
ip.addOptional('trueafternet',[],@(x) isnumeric(x)||iscell(x))
ip.addParamValue('blnSave',true,@islogical) 
ip.addParamValue('strFileAppend','',@ischar)
ip.addParamValue('Yl',[],@isnumeric)
ip.addParamValue('Xl',[],@isnumeric)
ip.addParamValue('strSaveFolder','',@ischar)
ip.addParamValue('blnObsHeight',true,@islogical)
ip.addParamValue('strSaveFilePrepend','',@ischar)
ip.addParamValue('swtCI','quantile',@(x) any(strcmp(x,{'quantile','sterr','BCa'})))
ip.addParamValue('proc',[],@isstruct)
ip.parse(varargin{:});

subjects = ip.Results.subjects;
reftype = ip.Results.reftype;
processing = ip.Results.processing;
nettype = ip.Results.nettype;
trueafternet = ip.Results.trueafternet;
blnsave = ip.Results.blnSave;
strFileAppend = ip.Results.strFileAppend;
Yl = ip.Results.Yl;
Xl = ip.Results.Xl;
strSaveFolder = ip.Results.strSaveFolder;
blnObsHeight = ip.Results.blnObsHeight;
strSaveFilePrepend = ip.Results.strSaveFilePrepend;
swtCI = ip.Results.swtCI;
proc = ip.Results.proc;


% blnsave = true;

addpath('..')
close all



switch nettype,
    case 'CanonicalCorrelation'
        nettype_nn = 'CCorr';
    case 'zeroLagCorr'
        nettype_nn = 'Corr';
    case 'absZeroLagCorr'
        nettype_nn = 'AbsCorr';
end

nonZeroSNR = cellfun(@(x) isempty(strfind(x,'T0.00')),subjects);

blntruecomp = ~isempty(trueafternet);
if blntruecomp,
    if ~iscell(trueafternet),
        tan = trueafternet;
        clear trueafternet
        trueafternet{1} = zeros(size(tan));
        trueafternet{2} = tan;
    end
    ntrialparts = 2;
end

if ~isempty(strfind(processing,'aveReg')),
    strFolderAppend = '_averaged';
elseif ~isempty(strfind(processing,'var')),
    strFolderAppend = ['_' regexp(processing,'var\d*\.*\d*','match','once')];
elseif ~strcmp(swtCI,'quantile')
    strFolderAppend = ['_' swtCI];
else
    strFolderAppend = '';
end

if isempty(strSaveFolder),
    saveFolder = 'Output_Plots/';
    savepath = [saveFolder nettype strFolderAppend strFileAppend '/'];
else
    savepath = strSaveFolder;
end


if blnsave,
    mkdir(savepath)
end

nsubjs = length(subjects);

%% extract C and density from agg_ebs variables

bs_median = zeros(2,nsubjs);
bs_ci = zeros(2,2,nsubjs);
obs_density = zeros(2,nsubjs);
for s=1:nsubjs,
    subject = subjects{s};

    loadpath = ['Output_Processed Correlation Networks/' reftype '/' processing '/' nettype '/wOuterBootstrap/'];
    w=load([loadpath 'workspace' '_S' subject '_BA' strFileAppend '.mat'],'agg','Nbsreps_outer');
    
    Nbsreps_outer = w.Nbsreps_outer;
    ba_nnets = length(w.agg);
    N = size(w.agg(1).C,1);

    if isempty(proc),
        webs=load([loadpath 'workspace' '_S' subject '_BA' strFileAppend '.mat'],'agg_ebs');

        ba_density = nan(ba_nnets,Nbsreps_outer);
        for j = 1:ba_nnets,
            for i=1:Nbsreps_outer,
                ba_density(j,i) = webs.agg_ebs{i}(j).density;
            end
        end
    else
        ba_density = proc(s).density.ba_ebs;
    end
    
    bs_median(:,s) = quantile(ba_density',0.5);
    obs_density(:,s) = [w.agg(:).density];
    switch swtCI,
        case 'quantile'
            bs_ci(:,:,s) = quantile(ba_density',[0.025 0.975]);
        case 'sterr'
            sterr = std(ba_density,[],2);
            bs_ci(:,:,s) = [obs_density(:,s)+norminv(0.025)*sterr obs_density(:,s)+norminv(0.975)*sterr]';
    end
end
clear w
%% plot before/after density

fig = figure(4);
if nsubjs>1,
    set(fig,'Position',[ 560   706   538   242])
else
    set(fig,'Position',[ 560   706   313   242])
end
set(fig,'Color','w')

if blnObsHeight,
    heights = obs_density;
else
    heights = bs_median;
end
lci = shiftdim(bs_ci(1,:,:),1);
uci = shiftdim(bs_ci(2,:,:),1);
h=bar(heights,1,'hist');
set(h,'FaceColor',[1 1 1]*.5)
set(h,'EdgeColor','none')
xd = get(h,'xdata');
if ~iscell(xd),
    xd = {xd};
end
xmiddles = cellfun(@(x)(x(1,:)+x(3,:))/2,xd,'UniformOutput',false);
xmiddles = cat(1,xmiddles{:});
hold on
h2=errorbar(xmiddles',(lci+uci)/2,(lci+uci)/2-lci,uci-(lci+uci)/2,'LineStyle','none','color','black','linewidth',2);
if blntruecomp,
    for i=1:ntrialparts,
        truedensity = sum(sum(trueafternet{i}))./(N*(N-1)); % scalar
        truedensity = truedensity*nonZeroSNR; % 1 x nsubjs

        scatter(xmiddles(:,i),truedensity,100,'kd','filled')
    end
end
uistack(h,'bottom')

hold off
grid off

% boxplot(ba_density')
set(gca,'XTickLabel',' ')
if ~isempty(Yl),
    ylim(Yl);
end
if ~isempty(Xl),
    xlim(Xl);
end

set(gca,'LineWidth',2)
set(gca,'Layer','top')
set(gca,'Box','off')
set(gca,'FontSize',14)


if blnsave,
    filename = [savepath strSaveFilePrepend 'PanSubj_' nettype_nn 'BA_densityBX'];
    rgb2cm(fig)
    export_fig([filename '.pdf'],'-pdf','-m1', fig)  
    export_fig([filename '.png'],'-png','-r1200','-m1', fig)  
end


