% Copyright (C) Emily Stephen 2014
function [node_h,edge_h,text_h]=sndf_plotnetwork(varargin)

ip=inputParser;
ip.addRequired('SNDFin',@isstruct);
ip.addRequired('brain',@isstruct)
ip.addRequired('C',@(x) isnumeric(x) || islogical(x)) % Connectivity matrix (nxn, 1/0 or true/false)
ip.addOptional('colorvals',[],@isnumeric) % Color/weight of edges (nxn, numeric). [] means plot red
ip.addParamValue('regions',[],@isstruct)
ip.addParamValue('reg_colors',[],@isnumeric)
ip.addParamValue('offset',[0 0 0],@isnumeric)
ip.addParamValue('bln2D',true,@islogical)
ip.addParamValue('cmap','jet',@(x) ischar(x)||isnumeric(x))
ip.addParamValue('colormax',[],@isnumeric)
ip.addParamValue('colorlims',[],@isnumeric)
ip.addParamValue('blnLabelElec',false,@islogical)
ip.parse(varargin{:})

SNDFin = ip.Results.SNDFin;
brain = ip.Results.brain; 
C = ip.Results.C; 
colorvals = squeeze(ip.Results.colorvals); 
offset = ip.Results.offset;
bln2D = ip.Results.bln2D;
cmap = ip.Results.cmap;
colormax = ip.Results.colormax;
colorlims = ip.Results.colorlims;
blnLabelElec = ip.Results.blnLabelElec;
regions = ip.Results.regions;
reg_colors = ip.Results.reg_colors;

set(gcf,'renderer','zbuffer')

electrodes = brain.tala.trielectrodes(SNDFin.vars.good_channels,:);
electrodes = electrodes + ones(size(electrodes,1),1)*offset;

[e1,e2] = find(C);
nedges = length(e1);
if ~isempty(colorvals),
    blnColor = true;
    
    colors = colorvals(sub2ind(size(C),e1,e2));
    [colors,ix] = sort(colors);
    e1 = e1(ix);
    e2 = e2(ix);
    
    if isempty(colorlims),
        if isempty(colormax),
            colormax = max(abs(colors));
        end
        if colormax==0,
            colormax = 1;
        end
        if isempty(colors)||min(colors)<0||min(colors)==colormax,
            colormin = -colormax;
        else
            colormin = min(colors);
        end
    else
        colormin = colorlims(1);
        colormax = colorlims(2);
    end
    
    colormap(cmap)
    cmap = colormap;
    ncolors = size(cmap,1);
else
    blnColor = false;
end

if bln2D,
    node_h=sndf_plotelectrodes(SNDFin,brain,false,offset,'bln2D',bln2D);
else
    node_h=sndf_plotelectrodes(SNDFin,brain,false,offset+[-1 0 0],'bln2D',bln2D);
end
edge_h=zeros(1,nedges);
for edge=1:nedges,
    i = e1(edge);
    j = e2(edge);
    if blnColor,
        color = colors(edge);
        colix = round((ncolors-1)*(color-colormin)./(colormax-colormin)) + 1;
        if bln2D,
            edge_h(edge)=line([electrodes(i,2) electrodes(j,2)],[electrodes(i,3) electrodes(j,3)],'color',cmap(colix,:));
        else
            edge_h(edge)=line([electrodes(i,1) electrodes(j,1)],[electrodes(i,2) electrodes(j,2)],[electrodes(i,3) electrodes(j,3)],'color',cmap(colix,:));
        end
    else
        if bln2D,
            edge_h(edge)=line([electrodes(i,2) electrodes(j,2)],[electrodes(i,3) electrodes(j,3)],'color','k');
        else
            edge_h(edge)=line([electrodes(i,1) electrodes(j,1)],[electrodes(i,2) electrodes(j,2)],[electrodes(i,3) electrodes(j,3)],'color','k');
        end
    end
end
set(edge_h,'linewidth',2)
set(node_h,'sizedata',10^2)
if ~isempty(regions),
    Nr = length(regions);
    if isempty(reg_colors),
        reg_colors = nan(Nr,3);
        for f=1:Nr,
            reg_colors(f,:) = regions(f).color;
        end
    end

    % Region locations
    elec_colors = zeros(length(electrodes),3);
    for f=1:Nr,
        [~,~,reg_good_channels] = intersect(regions(f).electrodes,SNDFin.ChLbl);
        elec_colors(reg_good_channels,:) = repmat(reg_colors(f,:),length(reg_good_channels),1);
    end
    set(node_h,'cdata',elec_colors)
end
uistack(node_h,'top')


if blnColor,
    colorbar('location','EastOutside'),caxis([colormin colormax])
end

text_h=zeros(1,size(electrodes,1));
if blnLabelElec,
    ChLbl = SNDFin.ChLbl;
    for e=1:size(electrodes,1),
        if bln2D,
            text_h(e)=text(electrodes(e,2)+.1,electrodes(e,3),ChLbl{e});
        else
            text_h(e)=text(electrodes(e,1),electrodes(e,2)+.1,electrodes(e,3),ChLbl{e});
        end
    end
end


