% Copyright (C) Emily Stephen 2014
function [regnames,region_nodeloc,node_h,edge_h,text_h] = sndf_plotnetwork_regions(varargin)

ip=inputParser;
ip.addRequired('SNDFin',@isstruct)
ip.addRequired('brain',@isstruct)
ip.addRequired('C',@(x) isnumeric(x) || islogical(x)) % Connectivity matrix (NrxNr, 1/0 or true/false)
ip.addRequired('regions',@isstruct)
ip.addOptional('colorvals',[],@isnumeric) % Color/weight of edges (NrxNr, numeric). [] means plot black
ip.addParamValue('offset',[0 0 0],@isnumeric)
ip.addParamValue('bln2D',true,@islogical)
ip.addParamValue('cmap','jet',@(x) ischar(x)||isnumeric(x))
ip.addParamValue('colormax',[],@isnumeric)
ip.addParamValue('colorlims',[],@isnumeric)
ip.addParamValue('reg_colors',[],@isnumeric)
ip.addParamValue('blnLabelReg',false,@islogical)
ip.parse(varargin{:})

SNDFin = ip.Results.SNDFin;
brain = ip.Results.brain; 
C = ip.Results.C; 
regions = ip.Results.regions;
colorvals = squeeze(ip.Results.colorvals); 
offset = ip.Results.offset;
bln2D = ip.Results.bln2D;
reg_colors = ip.Results.reg_colors;
cmap = ip.Results.cmap;
colormax = ip.Results.colormax;
colorlims = ip.Results.colorlims;
blnLabelReg = ip.Results.blnLabelReg;

set(gcf,'renderer','zbuffer')

electrodes = brain.tala.trielectrodes;
electrodes = electrodes + ones(size(electrodes,1),1)*offset;

Nr = size(C,1);

% Region locations
region_nodeloc = nan(Nr,3);
regnames = cell(1,Nr);
for f=1:Nr,
    regnames{f} = regions(f).name;

    if Nr==length(SNDFin.ChLbl),
        reg_good_channels = cellfun(@str2num,regions(f).electrodes);
    else
        reg_good_channels = cellfun(@str2num,intersect(regions(f).electrodes,SNDFin.ChLbl));
    end
    region_nodeloc(f,:) = mean(electrodes(reg_good_channels,:),1);
end

% Region Colors
if isempty(reg_colors),
    reg_colors = nan(Nr,3);
    for f=1:Nr,
        reg_colors(f,:) = regions(f).color;
    end
end

% Edge Colors
[e1,e2] = find(C);
nedges = length(e1);
if ~isempty(colorvals),
    blnColor = true;
    
    colors = colorvals(sub2ind(size(C),e1,e2));
    [colors,ix] = sort(colors);
    e1 = e1(ix);
    e2 = e2(ix);
    
    if isempty(colorlims),
        if isempty(colormax),
            colormax = max(abs(colors));
        end
        if colormax==0,
            colormax = 1;
        end
        if isempty(colors)||min(colors)<0||min(colors)==colormax,
            colormin = -colormax;
        else
            colormin = min(colors);
        end
    else
        colormin = colorlims(1);
        colormax = colorlims(2);
    end
    
    colormap(cmap)
    cmap = colormap;
    ncolors = size(cmap,1);
else
    blnColor = false;
end

% subplot(2,1,1)
% [reg_locs,areanames] = sndf_plotregions(SNDFin,brain,regions,offset,'bln2D',bln2D,'colors',reg_colors);
% subplot(2,1,2)
% node_h=sndf_scatter(region_nodeloc,'bln2D',bln2D,'colors',reg_colors,'blnfilled',true);
if bln2D,
    node_h=sndf_scatter(region_nodeloc,'bln2D',bln2D,'blnfilled',true);
else
    node_h=sndf_scatter(region_nodeloc+repmat([-1 0 0],Nr,1),'bln2D',bln2D,'blnfilled',true);
end
% legend(regnames)
edge_h=zeros(1,nedges);
for edge=1:nedges,
    i = e1(edge);
    j = e2(edge);
    if blnColor,
        color = colors(edge);
        colix = round((ncolors-1)*(color-colormin)./(colormax-colormin)) + 1;
        if bln2D,
            edge_h(edge)=line([region_nodeloc(i,2) region_nodeloc(j,2)],[region_nodeloc(i,3) region_nodeloc(j,3)],'color',cmap(colix,:));
        else
            edge_h(edge)=line([region_nodeloc(i,1) region_nodeloc(j,1)],[region_nodeloc(i,2) region_nodeloc(j,2)],[region_nodeloc(i,3) region_nodeloc(j,3)],'color',cmap(colix,:));
        end
    else
        if bln2D,
            edge_h(edge)=line([region_nodeloc(i,2) region_nodeloc(j,2)],[region_nodeloc(i,3) region_nodeloc(j,3)],'color','k');
        else
            edge_h(edge)=line([region_nodeloc(i,1) region_nodeloc(j,1)],[region_nodeloc(i,2) region_nodeloc(j,2)],[region_nodeloc(i,3) region_nodeloc(j,3)],'color','k');
        end
    end
end
set(edge_h,'linewidth',2)
set(node_h,'sizedata',10^2)
set(node_h,'cdata',reg_colors)
uistack(node_h,'top')

if blnColor,
    colorbar,caxis([colormin colormax])
end

text_h=zeros(1,size(length(regnames),1));
if blnLabelReg,
    for r=1:length(regnames),
        if bln2D,
            text_h(r)=text(region_nodeloc(r,2)+.1,region_nodeloc(r,3),regnames{r});
        else
            text_h(r)=text(region_nodeloc(r,1),region_nodeloc(r,2)+.1,region_nodeloc(r,3),regnames{r});
        end
    end
end
