% Copyright (C) Emily Stephen 2014
function SNDFout=below30Hz_preprocess(varargin)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Keeps good channels as defined by the good_channels variable in the vars
%   field. 
% High-pass filter (3rd order Butterworth Filter, zero-phase) at 0.1Hz to
%   remove drift (not in original paper).
% Low-pass filter (3rd order Butterworth Filter, zero-phase) below 30Hz.
% Then apply CAR.
%
% INPUT:
% SNDFin - the SNDF CNT struct to be processed
%
% OUTPUT:
% SNDFout - Preprocessed SNDF CNT struct
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Set input parameters
ip=inputParser;
ip.addRequired('SNDFin',@isstruct);
ip.addParamValue('SelectGoodChannels',true,@islogical);
ip.addParamValue('blnCAR',true,@islogical);
ip.addParamValue('CAR_sets',{},@iscell);
ip.parse(varargin{:})
SNDFin = ip.Results.SNDFin;

% Set other parameters
lp_freq = 30; %Hz
hp_freq = 0.1; %Hz
% notch1_freq = 60; %Hz
% notch2_freq = 120; %Hz

SNDFout = basic_preprocess(SNDFin,lp_freq,hp_freq,[],'SelectGoodChannels',ip.Results.SelectGoodChannels,'post_fs',200,'blnCAR',ip.Results.blnCAR,'CAR_sets',ip.Results.CAR_sets);
