% Copyright (C) Emily Stephen 2014
function [out,baseline_signal_corr,baseline_signal_uncorr,trial_corrpart,trial_uncorrpart]=simulatedData_ramps(subject,reftype,processings,sim,blnSpecs,blnSave,regions)


N = sim.N; 
Fs = sim.Fs;
ntrials = sim.ntrials;
T = sim.T; 
temp_corr = sim.temp_corr;
white_gain = sim.white_gain;
baseline_band = sim.baseline_band;
baseline_total_gain = sim.baseline_total_gain; 
trial_band = sim.trial_band;
trial_gain = sim.trial_gain; 
trial_electrode_gains = sim.trial_electrode_gains; % will be normalized so that max(sum(gain,2))=1;
trial_onsets_s = sim.trial_onsets_s;
silence_onsets_s = sim.silence_onsets_s;
trial_dur_s = sim.trial_dur_s;
baseline_corr_gain = sim.baseline_corr_gain;% will be normalized to have max 1

if nargin<5,
    blnSpecs = false;
end
if nargin<6,
    blnSave = true;
end

%specparams
sparams.tapers = [5 8];
sparams.pad = 0;
sparams.Fs = Fs;

disp(['Simulating data for subject ' subject])
tstart = tic;

if ~iscell(trial_electrode_gains),
    teg = trial_electrode_gains;
    clear trial_electrode_gains
    trial_electrode_gains{1} = zeros(size(teg));
    trial_electrode_gains{2} = teg;
end
ntrialparts = 2;


%% Normalize electrode gains

for i=1:ntrialparts,
    total_gainbyE = sum(abs(trial_electrode_gains{i}),1);
    if sum(total_gainbyE)~=0,
        trial_electrode_gains{i} = trial_electrode_gains{i}/max(total_gainbyE);
    end
end


%%
dt = 1/Fs;
ta  = [ 0 : T-1] * dt';  % time
nyq = Fs/2;
trial_dur_i = trial_dur_s * Fs;
sigdur = trial_dur_i/2;

trialsig_windows{1} = [0 sigdur-1];
trialsig_windows{2} = [sigdur trial_dur_i-1];

%% Create simulated signal and utterances

% pink noise
disp(['   Calculating pink_component...'])
pink_component = zeros(T,N);
wn = randn( T, N );
g = normpdf( ta, mean(ta), temp_corr/4 ); % envelope and also used as a filter set_4_sigma_to_be_duration_of_temporal_correlation
g=g(g>0);
for i=1:N,
    pink_component(:,i) = conv( wn(:,i), g, 'same' ); % filtering colors noise (use 'valid' if you're worried about transients)
end

pink_component = bsxfun(@rdivide,pink_component,std(pink_component,[],1));
if blnSpecs,
    [S_pink,f] = mtspectrumc(pink_component,sparams);
end

% white sensor noise
disp(['   Calculating white_component...'])
if white_gain~=0,
    white_component = randn(T,N)*sqrt(white_gain);
    if blnSpecs,
        [S_white] = mtspectrumc(white_component,sparams);
    end
else
    white_component = zeros(T,N);
    S_white = [];
end

% add baseline correlations for all time
disp(['   Calculating baseline_component...'])
if baseline_total_gain~=0,
    [b,a] = butter(4,baseline_band/nyq);
    % freqz(b,a,[],Fs)
    baseline_signal_corr = randn(T,1);
    baseline_signal_corr = filtfilt(b,a,baseline_signal_corr);
    baseline_signal_corr = baseline_signal_corr ./ std(baseline_signal_corr,[],1);
    baseline_signal_corr = sqrt(baseline_corr_gain)*baseline_signal_corr*ones(1,N);

    baseline_signal_uncorr = randn(T,N);
    baseline_signal_uncorr = filtfilt(b,a,baseline_signal_uncorr);
    baseline_signal_uncorr = bsxfun(@rdivide,baseline_signal_uncorr,std(baseline_signal_uncorr,[],1));
    baseline_signal_uncorr = sqrt(1-baseline_corr_gain)*baseline_signal_uncorr;

    baseline_component = sqrt(baseline_total_gain) * (baseline_signal_corr + baseline_signal_uncorr);
    if blnSpecs,
        [S_baseline] = mtspectrumc(baseline_component,sparams);
    end
else
    baseline_component = zeros(T,N);
    S_baseline = [];
end

% add signal in trials
disp(['   Calculating trial_component...'])
trial_onsets_i = nan(size(trial_onsets_s));
for t=1:ntrials,
    trial_onsets_i(t) = find(ta>=trial_onsets_s(t),1,'first');
end
if trial_gain~=0,

    if isfield(sim,'trial_window'),
        switch sim.trial_window,
            case 'gaussian',
                trial_window = normpdf((1:sigdur)'/Fs,trial_dur_s/4,trial_dur_s/20);
                trial_window = trial_window/max(trial_window);
            otherwise
                trial_window = ones(sigdur,1);
        end
    else
        trial_window = ones(sigdur,1);
    end

    for i=1:ntrialparts,
        nsubsigs{i} = size(trial_electrode_gains{i},1);
        total_gainbyE = sum(abs(trial_electrode_gains{i}),1);
        uncorr_rampdown{i} = sqrt(ones(sigdur,N)-(trial_window*total_gainbyE));
    end

    trial_uncorrpart = randn(T,N); % uncorrelated trial-band signal to balance the variance between trial/nontrial intervals
    trial_corrpart = zeros(T,N); 
    for t=1:ntrials,
        for i=1:ntrialparts,
            startind = trial_onsets_i(t)+trialsig_windows{i}(1); % put signal in the second half of the trial
            endind = trial_onsets_i(t)+trialsig_windows{i}(2);
            subcomponent = zeros(sigdur,N,nsubsigs{i});
            for s=1:nsubsigs{i},
                gains = trial_electrode_gains{i}(s,:); % 1 x N
                subsignal = randn(sigdur,1); % dur x 1

                subsignal = subsignal .* sqrt(trial_window);
                subcomponent(:,:,s) = subsignal * (sqrt(abs(gains)).*sign(gains)); % dur x N
            end
            corrpart = sum(subcomponent,3);
            uncorrpart = trial_uncorrpart(startind:endind,:).*uncorr_rampdown{i};
            trial_uncorrpart(startind:endind,:) = uncorrpart;
            trial_corrpart(startind:endind,:) = corrpart;
        end
    end
    
    [b,a] = butter(4,trial_band/nyq);
    trial_uncorrpart = filtfilt(b,a,trial_uncorrpart); 
    trial_corrpart = filtfilt(b,a,trial_corrpart); 
    trial_component = trial_uncorrpart+trial_corrpart; 
    gain_correct = std(trial_component,[],1);
    
    trial_uncorrpart = bsxfun(@rdivide,trial_uncorrpart,gain_correct);
    trial_corrpart = bsxfun(@rdivide,trial_corrpart,gain_correct);
    trial_component = bsxfun(@rdivide,trial_component,gain_correct);
    
    trial_uncorrpart = sqrt(trial_gain) * trial_uncorrpart;
    trial_corrpart = sqrt(trial_gain) * trial_corrpart;
    trial_component = sqrt(trial_gain) * trial_component;
    if blnSpecs,
        [S_trial] = mtspectrumc(trial_component,sparams);
    end
   
    % take out trial info for SNR calculation
    corrparts = zeros(sigdur,N,ntrials);
    uncorrparts = zeros(sigdur,N,ntrials);
    trials = zeros(sigdur,N,ntrials);
    for t=1:ntrials,
        startind = trial_onsets_i(t)+trial_dur_i-sigdur; % put signal in the second half of the trial
        endind = trial_onsets_i(t)+trial_dur_i-1;
        corrparts(:,:,t) = trial_corrpart(startind:endind,:);
        uncorrparts(:,:,t) = trial_uncorrpart(startind:endind,:);
        trials(:,:,t) = trial_component(startind:endind,:);
    end
    trial_corrvar = mean(var(corrparts,[],1),3);
    trial_uncorrvar = mean(var(uncorrparts,[],1),3);
        
else
    trial_component = zeros(T,N);
    S_trial = [];
    trial_corrvar = zeros(1,N);
    trial_uncorrvar = zeros(1,N);
end

% Total signal
disp(['   Calculating total...'])
simdata = pink_component + white_component + baseline_component + trial_component;
simdata = bsxfun(@rdivide,simdata,std(simdata,[],1)); % normalize to std 1
if blnSpecs,
    [S_total] = mtspectrumc(simdata,sparams);
end

SNR = trial_corrvar*trial_gain ./ (trial_uncorrvar*trial_gain + baseline_total_gain + white_gain + 1);
corrat = trial_corrvar*trial_gain./(baseline_corr_gain*baseline_total_gain);

if blnSave,
    %% create varfile
    disp(['   Creating var file...'])
    varfolder = ['Output_Preprocessed Data/SUBJECT' subject '/'];
    mkdir(varfolder)
    varfile = [varfolder 'SUBJECT' subject '_vars.mat'];


    vars.bad_channels = [];
    vars.good_channels = [1:N];

    vars.trial_corrvar = trial_corrvar;
    vars.trail_uncorrvar = trial_uncorrvar;
    vars.SNR = SNR;
    vars.corrat = corrat;

    vars.sim = sim;

    save(varfile,'-struct','vars','-v7.3');

    %% create region file
    disp(['   Creating region file...'])
    if nargin<7,
        regions(1) =  struct('name','R1','electrodes',{{'1','2','3'}},'color',[255 0 0]/255); % aMFg/46,9
        regions(2) =  struct('name','R2','electrodes',{{'4','5','6'}},'color',[0 255 0]/255); % aMFg/46,9
        regions(3) =  struct('name','R3','electrodes',{{'7','8','9'}},'color',[0 0 255]/255); % aMFg/46,9
    end

    save([varfolder 'SUBJECT' subject '_regions.mat'],'regions');

    %%
    disp(['   Creating cnt and dsc files...'])
    for p=1:length(processings),
        processing = processings{p};

        % create cnt
        savepath = ['Output_Preprocessed Data/SUBJECT' subject '/' reftype '/' processing];
        mkdir(savepath)


        cntfile = [savepath filesep 'ECOGS001' '_' processing '_cnt.mat'];

        if strcmpi(reftype,'Physical'),
            blnCAR = false;
        elseif strcmpi(reftype,'CAR'),
            blnCAR = true;
        else
            error('Unrecognized Reftype')
        end

        switch processing
            case 'broad1to200'
                preprocfun = @(x) broad1to200_preprocess(x,'blnCAR',blnCAR);
            case 'above30Hz'
                preprocfun = @(x) above30Hz_preprocess(x,'blnCAR',blnCAR);
            case 'below30Hz'
                preprocfun = @(x) below30Hz_preprocess(x,'blnCAR',blnCAR);
            case 'highgamma'
                preprocfun = @(x) highgamma_preprocess(x,'blnCAR',blnCAR);
        end

        data = struct('SampValues',simdata,...
                      'SampFreq',Fs,...
                      'ChLbl',{cellfun(@num2str,num2cell(1:N),'UniformOutput',false)},...  
                      'SubjectID',['SUBJECT' subject],...
                      'SampTimes',1/Fs,...
                      'FragLengths',T,...
                      'DataUnits','microVolts',...
                      'TimeUnits','s',...
                      'Log',cell(1));
        data.Log{1} = subject;
        data.Log{2,1}=mfilename;
        data.Log{2,2}=datestr(clock);

        [data.vars] = vars;

        [data] = preprocfun(data);

        save(cntfile,'-struct','data','-v7.3');

    %     data_check_v2(cntfile)

        % create dsc_utterances
        dscfile = [savepath '/ECOGS001_manualUtt_wcnt_dsc.mat'];

        [dsc_wcnt] = utterances_fromCSV_wSegValues(trial_onsets_s+trial_dur_s/2,cntfile,dscfile,trial_dur_s/2,trial_dur_s/2);
    %     data_check_v2(dscfile)
    end

    %% create dsc_silences
    disp(['   Creating silence file...'])
    silencefile = [varfolder '/ECOGS001_silenceOnOff_dsc.mat'];

    EvtTimes = [silence_onsets_s; silence_onsets_s+1];
    EvtID = [ones(size(silence_onsets_s)); ones(size(silence_onsets_s))*2];

    [sortedEvtTimes,I] = sort(EvtTimes);
    sortedEvtID = EvtID(I);

    EvtLbl = {'Onset','Offset'};
    silenceSNDF = struct('EvtTimes',sortedEvtTimes,...
                     'EvtID',sortedEvtID,...
                     'EvtLbl',{EvtLbl},...
                     'vars',data.vars,...
                     'TimeUnits','s',...
                     'Log',{data.Log});


    % Add a Log entry
    pos=size(silenceSNDF.Log,1)+1;
    silenceSNDF.Log{pos,1}=mfilename;
    silenceSNDF.Log{pos,2}=datestr(clock);

    save(silencefile,'-struct','silenceSNDF','-v7.3');

    %% empty brain file
    disp(['   Creating brain file...'])
    brain.tala.trielectrodes = [zeros(N,1) sin(2*pi*(1:N)'/N) cos(2*pi*(1:N)'/N)];
    save(['Output_Preprocessed Data/AMC_' subject '_brain.mat'],'-struct','brain','-v7.3')
end
%%
disp(['   Creating output struct...'])
out.ta = ta;
out.pink_component = pink_component;
out.white_component = white_component;
out.baseline_component = baseline_component;
out.trial_component = trial_component;
out.simdata = simdata;
if blnSpecs,
    out.f = f;
    out.S_pink = S_pink;
    out.S_white = S_white;
    out.S_baseline = S_baseline;
    out.S_trial = S_trial;
    out.S_total = S_total;
end
out.trial_dur_i = trial_dur_i;
out.trial_onsets_i = trial_onsets_i;
out.SNR = SNR;
out.corrat = corrat;

toc(tstart)
